///////////////////////////////////////////////////////////////////////////////
//	Copyright (c) 2003, 2004, 2005, Intel Corporation
//	All rights reserved.
//
//	Redistribution and use in source and binary forms, with or without
//	modification, are permitted provided that the following conditions are met:
//		* Redistributions of source code must retain the above copyright notice,
//		  this list of conditions and the following disclaimer.
//		* Redistributions in binary form must reproduce the above copyright
//		  notice, this list of conditions and the following disclaimer in the
//		  documentation and/or other materials provided with the distribution.
//		* Neither the name of the Intel Corporation nor the names of its
//		  contributors may be used to endorse or promote products derived from
//		  this software without specific prior written permission.
//
//	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS AS IS
//	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
//	IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
//	ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
//	LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
//	CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
//	SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
//	INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
//	CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
//	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
//	POSSIBILITY OF SUCH DAMAGE.
//
///////////////////////////////////////////////////////////////////////////////
//	File Name:		CDlgcTimer.h
//
//	Description:	
//
//	Declared in this file:
//		#define ::_2_Timer_h_H
//		#define ::AFX_TIMER_H__E463B862_3FFE_11D5_A932_00D0B789E9BE__INCLUDED_
//		class DlgcFW2::ADlgcEventHandler
//		class DlgcFW2::CDlgcLogger
//		class DlgcFW2::timer_info
//		#define ::_MMRESULT_
//		typedef DlgcFW2::MMRESULT
//		typedef DlgcFW2::TIMER_LIST
//		class DlgcFW2::CDlgcTimer
//
//	Implemented in this file:
//		char * ::_2_Timer_h
//		DlgcFW2::timer_info::timer_info(ADlgcEventHandler *pD, int c, CDlgcEvent *event)
//		DlgcFW2::timer_info::timer_info(ADlgcEventHandler *pD=NULL, int c=-1, int e=DLGC_TIMER)
//		bool DlgcFW2::timer_info::<(const timer_info &rhs)
//		bool DlgcFW2::timer_info::>(const timer_info &rhs)
//		bool DlgcFW2::timer_info::==(const timer_info &rhs)
//		timer_info * DlgcFW2::timer_info::=(timer_info &rhs)
//		void DlgcFW2::timer_info::Cancel()
//		bool DlgcFW2::timer_info::IsCanceled()
//		CDlgcLogger * DlgcFW2::CDlgcTimer::Logger()
//		void DlgcFW2::CDlgcTimer::Logger(CDlgcLogger *logger)
//
///////////////////////////////////////////////////////////////////////////////
//	Revisions:
//	Date 		Initial		Description
//	---- 		-------		-----------
//	05/07/01	DMH			1. -- Initial Version --
//	01/16/02	GAT			2. Changed to use ADlgcEventTarget in place of ADlgcDev
//	01/16/02	GAT			3. Renamed from CTimer to CDlgcTimer
//	10/1/03		ATS			Updated comments
//	05/07/2004	GAT			Added a canceled flag to timer_info
//							Changed return type of CDlgcTimer AddDevice functions
//	06/15/2004	DMH			Added m_NumberOfSetTimers variable, added m_ to Period variable
//	01/10/2005	GAT			Updated Declared & Implemented sections
///////////////////////////////////////////////////////////////////////////////

#ifndef _2_Timer_h_H
#define _2_Timer_h_H

#ifndef lint
static char    *_2_Timer_h = "@(#) %filespec: Timer.h-1 %  (%full_filespec: Timer.h-1:incl:2 %)";
#endif

#if !defined(AFX_TIMER_H__E463B862_3FFE_11D5_A932_00D0B789E9BE__INCLUDED_)
	#define AFX_TIMER_H__E463B862_3FFE_11D5_A932_00D0B789E9BE__INCLUDED_

	#if _MSC_VER > 1000
		#pragma once
	#endif // _MSC_VER > 1000

	#pragma warning(disable: 4786)

	#ifdef LINUX_FW
		#include <signal.h>
		#include <sys/time.h>
		#include <math.h>
	#else
		#include <windows.h>
	#endif

	#include <list>

	#include <boost/thread/mutex.hpp>

	#include "DlgcEventDefs.h"
    #include "CDlgcEvent.h"
    
namespace DlgcFW2
{
	class ADlgcEventHandler;
	class CDlgcLogger;

///////////////////////////////////////////////////////////////////////////////
//	Class:		DlgcFW2::timer_info
//
//	Description:
//
//	Notes:		
//
///////////////////////////////////////////////////////////////////////////////
//	Revisions:
//	Date 		Initial		Description
//	---- 		-------		-----------
//	10/1/03 	ATS			Initial Version
//	05/07/2004	GAT			Added a canceled flag
///////////////////////////////////////////////////////////////////////////////
	class timer_info
	{

	public:

		ADlgcEventHandler* pDevice;	    // Device requesting timer.
		int Count;			            // Number of periodic timers before timer request expires.
	    CDlgcEvent *EventObj;			// Event (with data) to fire when timer request expires
		int Event;			            // Event to fire when timer request expires
		bool canceled;

		//
	    //  Constructors
		//
	    timer_info (ADlgcEventHandler * pD, int c , CDlgcEvent * event):
	        pDevice (pD), Count (c), EventObj (event), canceled(false)
	    {
            Event = -1;
	    }
		timer_info(ADlgcEventHandler* pD = NULL, int c= -1, int e=DLGC_TIMER):
		    pDevice(pD), Count(c), Event(e), canceled(false)
		{
             EventObj = NULL;
		}

		//
		//  Overload the GreaterThan, LessThan, and EqualTo Operators
		//  so this structure can be sorted in an STL list.
		//
		bool operator < (const timer_info & rhs)
		{

			return(this->Count < rhs.Count);
		}    

		bool operator > (const timer_info & rhs)
		{

			return(this->Count > rhs.Count);
		}

		bool operator == (const timer_info & rhs)
		{

			return(this->pDevice == rhs.pDevice);
		}

		timer_info* operator = (timer_info & rhs)
		{

			this->pDevice = rhs.pDevice;
			this->Count   = rhs.Count;

			return(this);

		}

		void Cancel() { canceled = true; }
		bool IsCanceled() { return canceled; }

	};

#ifndef LINUX_FW
#ifndef _MMRESULT_
#define _MMRESULT_
	typedef UINT                MMRESULT;
#endif
#endif

	typedef std::list<timer_info*> TIMER_LIST;


	/*
	 *  The CTimer object will create a periodic Multi-Media Timer that will
	 *  expire every 100 ms by default.  Each time the MMTimer expires, the Callback
	 *  function will traverse the TimerList and decrement each timer_info's Count by one.
	 *
	 *  Once the timer_info's Count is Zero (or less than Zero), the ADlgcDev's timer 
	 *  has expired and a TIMER event will get posted to the SRL Queue.
	 *  
	 *  The one parameter CTimer constructor will allow the default time period to be
	 *  changed if the user wants to change the MMTimer resolution.
	 */

///////////////////////////////////////////////////////////////////////////////
//	Class:		DlgcFW2::CDlgcTimer
//
//	Description:
//
//	Notes:		
//
///////////////////////////////////////////////////////////////////////////////
//	Revisions:
//	Date 		Initial		Description
//	---- 		-------		-----------
//	10/1/03 	ATS			Initial Version
//	05/07/2004	GAT			Changed return type of AddDevice functions
//	06/15/2004	DMH			Added m_NumberOfSetTimers variable, added m_ to Period variable
///////////////////////////////////////////////////////////////////////////////
	class CDlgcTimer
	{

	public:

		CDlgcTimer();	   // Start a periodic MMTimer with the default time.
		CDlgcTimer(int p); // Start a periodic MMTimer with period 'p'.
		virtual ~CDlgcTimer();	   // Destroy MMTimer and TimerList.

		timer_info* AddDevice( ADlgcEventHandler* pD); //  Add timer_info for this object to TimerList.
        timer_info* AddDevice( ADlgcEventHandler* pD, int delay, CDlgcEvent* eventObj);
		timer_info* AddDevice( ADlgcEventHandler* pD, int delay, int event);
		void RemoveDevice (ADlgcEventHandler* pD); // Take timer_info for this object off the list.

#ifndef LINUX_FW
		void CheckForExpirations ();  // Traverse TimerList Posting events for expired Devices.
#endif

		// Logging
		void LogMessage( const char* functionName, const char* stateDesc, const char* logMessage);


	private:
		int m_Period;			   // Default Period Timer Length.
		int m_NumberOfSetTimers;
#ifndef LINUX_FW
		MMRESULT   MMTimer;	   // Multi-media Timer
		TIMER_LIST TimerList;  // List of Devices using this timer.
		boost::mutex _criticalSection;
#endif

		CDlgcLogger* _logger;
		CDlgcLogger* Logger()
		{
			return _logger;
		};
		void Logger(CDlgcLogger* logger)
		{
			_logger = logger;
		};

	};
}  // end namespace

#endif // !defined(AFX_TIMER_H__E463B862_3FFE_11D5_A932_00D0B789E9BE__INCLUDED_)
#endif
