///////////////////////////////////////////////////////////////////////////////
//  Copyright (c) 2005, Intel Corporation
//  All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without
//  modification, are permitted provided that the following conditions are met:
//      * Redistributions of source code must retain the above copyright notice,
//        this list of conditions and the following disclaimer.
//      * Redistributions in binary form must reproduce the above copyright
//        notice, this list of conditions and the following disclaimer in the
//        documentation and/or other materials provided with the distribution.
//      * Neither the name of the Intel Corporation nor the names of its
//        contributors may be used to endorse or promote products derived from
//        this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS AS IS
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
//  POSSIBILITY OF SUCH DAMAGE.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef LINUX
#include <unistd.h>
#include <netdb.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <string.h>
extern int h_errno;
#else
#include <windows.h>
#include <winsock.h>
#endif

#include <stdio.h>

char *
LocalHostIPAddr (char *ipAddrBuf, size_t sizeof_ipAddrBuf)
{
#ifndef LINUX
  WORD wVersionRequested = MAKEWORD (2, 2);
  WSADATA wsaData;
  HOSTENT *lpHost = NULL;
#else
  struct hostent *lpHost = NULL;
#endif

  struct sockaddr_in addrstr;
  int rc;
  char hostname[256];

#ifndef LINUX
  if ((rc = WSAStartup (wVersionRequested, &wsaData)) != 0)
    {
      printf ("Could not find a useable winsock.dll\n");
      return NULL;
    }
#endif

  if ((rc = gethostname (hostname, 128)) != 0)
    {
      printf ("gethostname() returned error code %d\n", rc);
      return NULL;
    }

  if ((lpHost = gethostbyname (hostname)) == NULL)
    {
#ifdef LINUX
      printf ("gethostbyname failed: %s", hstrerror (h_errno));
#else
      printf ("gethostbyname failed: %d", WSAGetLastError ());
#endif
      return NULL;
    }

  if (lpHost->h_addr_list[0] == NULL)
    {
      printf ("TCP/IP not configured\n");
      return NULL;
    }

  memcpy (&(addrstr.sin_addr), lpHost->h_addr_list[0], lpHost->h_length);
  strncpy (ipAddrBuf, inet_ntoa (addrstr.sin_addr), sizeof_ipAddrBuf);

  return ipAddrBuf;
}
