///////////////////////////////////////////////////////////////////////////////
//  Copyright (c) 2005, Intel Corporation
//  All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without
//  modification, are permitted provided that the following conditions are met:
//      * Redistributions of source code must retain the above copyright notice,
//        this list of conditions and the following disclaimer.
//      * Redistributions in binary form must reproduce the above copyright
//        notice, this list of conditions and the following disclaimer in the
//        documentation and/or other materials provided with the distribution.
//      * Neither the name of the Intel Corporation nor the names of its
//        contributors may be used to endorse or promote products derived from
//        this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS AS IS
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
//  POSSIBILITY OF SUCH DAMAGE.
//
///////////////////////////////////////////////////////////////////////////////

#include <signal.h>
#include <stdio.h>

// External references
extern void CancelTimer ();

// Globals 
int GapplicationRunning;

//  Interrupt handler callbacks
#ifdef LINUX
void
intr_hdlr (int signal_number)
#else
void
intr_hdlr ()
#endif
{
  CancelTimer ();
  GapplicationRunning = 0;
#ifdef LINUX
  // The interrupt handler itself just sets a flag, displays signal number (see
  // man pages, section 7 for signal on Linux)
  printf
    ("Interrupt received. Timer canceled. Signal number is %d.  Hit CR to end program\n",
     signal_number);
#else
  printf ("Interrupt received. Timer canceled.  Hit CR to end program.\n");
#endif
}

// Set interrupt handler for interrupt signals
void
enable_int_handlers (void)
{

#ifdef LINUX
  signal (SIGHUP, intr_hdlr);
  signal (SIGQUIT, intr_hdlr);
  signal (SIGINT, intr_hdlr);
  signal (SIGTERM, intr_hdlr);
#else
  signal (SIGINT, (void (__cdecl *) (int)) intr_hdlr);
  signal (SIGTERM, (void (__cdecl *) (int)) intr_hdlr);
#endif

  GapplicationRunning = 1;
}
