// SharedMemory.h: interface for the CSharedMemory class.
//
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
//
//                       HISTORY OF CHANGE
//
//  DATE      WHO     VERSION     DESCRIPTION
// -------------------------------------------------------------------
// 11/07/00   DMH     1.1      Initial Version
// 11/07/00   DMH     1.2      Added Change History
//////////////////////////////////////////////////////////////////////

#ifdef WINNT

#if !defined(AFX_SHAREDMEMORY_H__86467BA6_5AFA_11D3_863D_00A0244A9CA7__INCLUDED_)
#define AFX_SHAREDMEMORY_H__86467BA6_5AFA_11D3_863D_00A0244A9CA7__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <process.h>
#include "secdesc.h"

class CSharedMemory
{
public:
	enum
	{
		// Return values for open().
		MEM_SUCCESS				= 0,
		MEM_ERROR_ALREADY_OPEN	= 1,
		MEM_ERROR_NAME			= 2,
		MEM_ERROR_DATASIZE		= 3,
		MEM_ERROR_MUTEX_CREATE	= 4,
		MEM_ERROR_2_ALREADY     = 5,
		MEM_ERROR_HANDLE_CREATE = 6,
		MEM_ERROR_MAP_SIZE_TAG  = 7,
		MEM_ERROR_DATASIZE_MATCH= 8,
		MEM_ERROR_NO_PTR_TO_DATA= 9
	};
	CSharedMemory()
	{
		// Set up security attributes for all agent interface related objects
		
		static EXPLICIT_ACCESS ea;
		static CSid sidEveryone(CSid::WST_EVERYONE);
		static CTrustee trEveryone(TRUSTEE_IS_WELL_KNOWN_GROUP, sidEveryone);
		
		ea = CExplicitAccess(GENERIC_ALL|STANDARD_RIGHTS_ALL|SPECIFIC_RIGHTS_ALL, SET_ACCESS, NO_INHERITANCE,
			trEveryone);
		
		// Create a new ACL and set the EA entries in it
		static CAcl acl;
		if(acl.SetEntriesInAcl(1, &ea) == ERROR_SUCCESS)
		{
			
			// Initialize a security descriptor and add our ACL to it  
			static CSecurityDescriptor sd;
			if(sd.SetSecurityDescriptorDacl(
				TRUE,     // fDaclPresent flag   
				acl, 
				FALSE))   // not a default DACL 
			{
				
				// Initialize a security attributes structure.
				m_SecAttr = new CSecurityAttributes (sd, FALSE);	
			}
		}  // end if ACL created
		
		// Create an event that indicates whether the connection
		// is open or not.
		m_hClosed = CreateEvent(m_SecAttr, TRUE, TRUE, NULL);
	};
	virtual ~CSharedMemory()
	{
		Close();
		CloseHandle(m_hClosed);
	};
	
	
	int Open(char* sName, int nDataSize, int nTimeOut = INFINITE)
	{
		
		// The connection must be closed before it can be opened.
		if (WaitForSingleObject(m_hClosed, 0) == WAIT_OBJECT_0)
		{
			// The name may not exceed MAX_PATH, we substract 10 because we
			// add some strings to the name in some code.
			if (strlen(sName) != 0 && strlen(sName) < MAX_PATH - 10)
			{
				// The datasize must be larger than 0.
				if (nDataSize > 0)
				{
					// The following mutexes can indicate 4 things:
					// - No instance of this shared memory class was created.
					// - The first instance of this class was created.
					// - The second instance of this shared memory class was created.
					// - Both instances were created.
					char sMutex0	[MAX_PATH];
					char sMutex1	[MAX_PATH];
					strcpy(sMutex0	, sName);
					strcpy(sMutex1	, sName);
					strcat(sMutex0	, "Mutex0");
					strcat(sMutex1	, "Mutex1");
					
					m_hSharedMemoryMutex[0] = CreateMutex(m_SecAttr, FALSE, sMutex0);
					m_hSharedMemoryMutex[1] = CreateMutex(m_SecAttr, FALSE, sMutex1);
									
					if (m_hSharedMemoryMutex[0] && m_hSharedMemoryMutex[1])
					{
						// Only two instances of this class (with this name) may reside on
						// one system. 
						HANDLE hWait[2] = {m_hSharedMemoryMutex[0], m_hSharedMemoryMutex[1]};
						DWORD dwResult = WaitForMultipleObjects(2, hWait, FALSE, 0);
					
						// hWait[1] is an abandoned mutex object.  Why???
						// Need to understand this.
						if (dwResult == WAIT_OBJECT_0 || dwResult == (WAIT_OBJECT_0 + 1) ||
							dwResult == WAIT_ABANDONED_0 || dwResult == (WAIT_ABANDONED_0 + 1))
						{
							char sName0		[MAX_PATH];
							strcpy(sName0	, sName);
							strcat(sName0	, "0");
							
							if ((m_hSharedMemory = CreateFileMapping(	(HANDLE)0xFFFFFFFF,
								m_SecAttr,
								PAGE_READWRITE,
								0,
								sizeof(int) + nDataSize,
								sName0)) != NULL)
							{
								bool bFileMappingAlreadyExists = (GetLastError() == ERROR_ALREADY_EXISTS);
								m_bAppIsStarted = bFileMappingAlreadyExists;

								// Now map a pointer to the size tag in the shared memory.
								m_pSize = (int*)MapViewOfFile(	m_hSharedMemory,
									FILE_MAP_ALL_ACCESS,
									0,
									0,
									sizeof(int));
								if (m_pSize)
								{
									bool bSharedMemorySizeOk = false;
									if (bFileMappingAlreadyExists)
									{
										// We will check if the size of the memory block is of the
										// same size as the block that was already allocated by another
										// instance of the shared memory class.
										// The size of the memory block is saved in the first integer
										// at the specified shared memory address.
										if (*m_pSize == nDataSize)
											bSharedMemorySizeOk = true;
									}
									else
									{
										// The memory was not allocated by another instance so we
										// have the honors to allocate it. This means also that we should
										// set the size of the memory that we have allocated in the first
										// integer of the shared memory space.
										*m_pSize = nDataSize;
										bSharedMemorySizeOk = true;
									}
									if (bSharedMemorySizeOk)
									{
										m_pSharedMemory = (BYTE*)MapViewOfFile(	m_hSharedMemory,
											FILE_MAP_ALL_ACCESS,
											0,
											0,
											nDataSize);
										if (m_pSharedMemory)
										{
											// Move the pointer a little further so that it does not point to
											// the size tag, but to the address of the data that we want to share.
											m_pSharedMemory += sizeof(int);;
											return MEM_SUCCESS;
											
										}
										else
										{
											// We could not get a pointer to the actual data.
											UnmapViewOfFile(m_pSharedMemory);
											return MEM_ERROR_NO_PTR_TO_DATA;
										}
									}
									else
									{
										// The datasize of the already allocated memory, and the size of this
										// instance do not match.
										UnmapViewOfFile(m_pSize);
										return MEM_ERROR_DATASIZE_MATCH;
									}
								}
								else
								{
									
									// We could not map to the integer that contains the size of the memory block.
									CloseHandle(m_hSharedMemory);
									return MEM_ERROR_MAP_SIZE_TAG;
								}
							}
							else
							{
								// The memory handles could not be created.
								return MEM_ERROR_HANDLE_CREATE;
							}
						}
						else
						{
							// There was no mutex available, this can mean that there are
							// already two instances of this object with the same name
							// in use on this system.
							CloseHandle(m_hSharedMemoryMutex[0]);
							CloseHandle(m_hSharedMemoryMutex[1]);
							return MEM_ERROR_2_ALREADY;
						}
					}
					else
					{
						// The mutexes could not be created.
						return MEM_ERROR_MUTEX_CREATE;
					}
				}
				else
				{
					// The datasize is not > 0.
					return MEM_ERROR_DATASIZE;
				}
			}
			else
			{
				// The name of the shared memory is not valid, or the datasize is not larger than 0.
				return MEM_ERROR_NAME;
			}
		}
		else
		{
			// This instance is already open.
			return MEM_ERROR_ALREADY_OPEN;
		}
	};

	
	void Close()
	{
		if (WaitForSingleObject(m_hClosed, 0) == WAIT_TIMEOUT)
		{
			// Indicate that this instance is closed
			SetEvent(m_hClosed);

			UnmapViewOfFile(m_pSize);
			
			m_pSharedMemory -= sizeof(int);
			UnmapViewOfFile(m_pSharedMemory);
			
			CloseHandle(m_hSharedMemory);
			
			CloseHandle(m_hSharedMemoryMutex[0]);
			CloseHandle(m_hSharedMemoryMutex[1]);			
		}
	};


private:
	
	// The two mutexes will indicate which instance is already available.
	HANDLE			m_hSharedMemoryMutex[2];
	HANDLE			m_hSharedMemory;
	int*			m_pSize;
	HANDLE			m_hClosed;			// This handle indicates whether this instance
										// is open or closed.
		
public:
	CSecurityAttributes *m_SecAttr;
	BYTE*				m_pSharedMemory;  // Ptr to beginning of shared mem block
	bool				m_bAppIsStarted;
};

#endif // !defined(AFX_SHAREDMEMORY_H__86467BA6_5AFA_11D3_863D_00A0244A9CA7__INCLUDED_)
#endif
