///////////////////////////////////////////////////////////////////////////////
//  Copyright (c) 2003, 2004, 2005, Intel Corporation
//  All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without
//  modification, are permitted provided that the following conditions are met:
//      * Redistributions of source code must retain the above copyright notice,
//        this list of conditions and the following disclaimer.
//      * Redistributions in binary form must reproduce the above copyright
//        notice, this list of conditions and the following disclaimer in the
//        documentation and/or other materials provided with the distribution.
//      * Neither the name of the Intel Corporation nor the names of its
//        contributors may be used to endorse or promote products derived from
//        this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS AS IS
//  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
//  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
//  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
//  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
//  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
//  POSSIBILITY OF SUCH DAMAGE.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef LINUX

#include <signal.h>
#include <sys/time.h>
#include <math.h>

#else

#include <windows.h>
#include <stdio.h>

// Global for Windows timer
MMRESULT GmmTimer;

#endif

void
OnTimerExpired ()
{
  // OS-neutral actions on callback would take place here 
  printf ("Timer expired...\n");
}


// Callbacks differ, but call a common neutral function
#ifdef LINUX
void
TimerCallback (int signum)
{
  OnTimerExpired ();
}
#else

void CALLBACK
TimerCallback (UINT uID, UINT uMsg, DWORD dwUser, DWORD dw1, DWORD dw2)
{
  OnTimerExpired ();
}
#endif

InitTimer (long timerPeriod)
{

#ifdef LINUX
  int rc;
  struct timeval timer_val;
  struct timeval interval_timer_val;
  struct itimerval itval;
  double msecs;

  // Specify signal handler for SIGALRM that timer will generate
  signal (SIGALRM, TimerCallback);

  // Set up time intervals for setitimer()
  // Time interval in is in milliseconds, but need to break it up into 
  // whole seconds and microseconds, which setitimer likes.  
  // Set both initial and interval values.

  timer_val.tv_sec = (long) floor (timerPeriod / 1000.0);   // Whole secs over 1000 ms
  timer_val.tv_usec = 1000 * (timerPeriod % 1000);  // microseconds remaining

  interval_timer_val.tv_sec = (long) floor (timerPeriod / 1000.0);
  //printf("sec value is %d\n", interval_timer_val.tv_sec);
  interval_timer_val.tv_usec = 1000 * (timerPeriod % 1000);
  //printf("Usec value is %d\n", interval_timer_val.tv_usec);

  itval.it_interval = interval_timer_val;
  itval.it_value = timer_val;

  rc = setitimer (ITIMER_REAL, &itval, (struct itimerval *) 0);

  if (rc != 0)

#else // Windows

  //static char * userData="User Data";
  DWORD userData = 999;
  GmmTimer = 0;

  GmmTimer = timeSetEvent (timerPeriod,
                           10, TimerCallback, userData, TIME_PERIODIC);
  if (GmmTimer == 0)
#endif
    {
      printf ("CTimer failed to create Timer.\n");
    }
  else
    {
      printf ("Periodic Timer ( timerPeriod = %d ) set\n\n", timerPeriod);
    }
}


void
CancelTimer ()
{

#ifdef LINUX
  // Cancel timer, and ignore any further alarm signals
  (void) alarm (0);
  signal (SIGALRM, SIG_IGN);
#else
  timeKillEvent (GmmTimer);
#endif
}
